#include "Initialisierung.h"
#include <ToolTip.h>
#include <Bildschirm.h>
#include <MausEreignis.h>
#include <DateiSystem.h>
#include <Fenster.h>
#include <Fortschritt.h>
#include <Liste.h>
#include "Globals.h"

Knopf* initKnopf(int x, int y, int br, int h, __int64 style, char* titel)
{
	Knopf* ret = uiFactory.createKnopf(uiFactory.initParam);
	ret->addStyle(style);
	ret->setPosition(x, y);
	ret->setSize(br, h);
	ret->setText(titel);
	return ret;
}

KontrollKnopf* initKontrollKnopf(int x, int y, int br, int h, __int64 style, char* txt)
{
	KontrollKnopf* ret = uiFactory.createKontrollKnopf(uiFactory.initParam);
	ret->addStyle(style);
	ret->setText(txt);
	ret->setSText(txt);
	ret->setPosition(x, y);
	ret->setSize(br, h);
	return ret;
}

Fenster* initFenster(int x, int y, int br, int h, __int64 style, char* titel)
{
	Fenster* ret = uiFactory.createFenster(uiFactory.initParam);
	ret->addStyle(style);
	ret->setPosition(x, y);
	ret->setSize(br, h);
	if (ret->hatStyle(Fenster::Style::Titel))
	{
		ret->setTitel(titel);
	}
	return ret;
}

TextFeld* initTextFeld(int x, int y, int br, int h, __int64 style, char* txt)
{
	TextFeld* ret = uiFactory.createTextFeld(uiFactory.initParam);
	ret->addStyle(style);
	ret->setText(txt);
	ret->setPosition(x, y);
	ret->setSize(br, h);
	return ret;
}

BildZ* initBildZ(int x, int y, int br, int h, __int64 style, Bild* b)
{
	BildZ* ret = uiFactory.createBildZ(uiFactory.initParam);
	ret->setStyle(style);
	ret->setPosition(x, y);
	ret->setSize(br, h);
	if (b)
		ret->setBildZ(b);
	return ret;
}

AuswahlBox* initAuswahlBox(int x, int y, int br, int h, __int64 style, std::initializer_list< char* > values)
{
	AuswahlBox* ret = uiFactory.createAuswahlBox(uiFactory.initParam);
	ret->addStyle(style);
	ret->setPosition(x, y);
	ret->setSize(br, h);
	for (auto i = values.begin(); i != values.end(); i++)
		ret->addEintrag(*i);
	return ret;
}

ObjTabelle* initObjTabelle(int x, int y, int br, int h, __int64 style, std::initializer_list< OBJTabelleSpalteIni > spalten, int berschriftHhe)
{
	ObjTabelle* ret = uiFactory.createObjTabelle(uiFactory.initParam);
	ret->addStyle(style);
	ret->setPosition(x, y);
	ret->setSize(br, h);
	for (auto i = spalten.begin(); i != spalten.end(); i++)
	{
		ret->addSpalte(i->name);
		ret->setSpaltenBreite(i->name, i->breite);
		if ((style | ObjTabelle::Style::SpaltenBreiteMin) == style)
			ret->setMinSpaltenBreite(i->name, i->minBreite);
		if ((style | ObjTabelle::Style::SpaltenBreiteMax) == style)
			ret->setMaxSpaltenBreite(i->name, i->maxBreite);
		if (berschriftHhe)
		{
			if (ret->getZeilenNummer("berschrift") < 0)
			{
				ret->addZeile(0, "berschrift");
				ret->setZeilenHeight(0, 20);
			}
			ret->setZeichnungZ(i->name, "berschrift", initTextFeld(0, 0, i->breite, 20, TextFeld::Style::Text | TextFeld::Style::Center, i->name));
		}
	}
	return ret;
}

LDiag* initLinienDiagramm(int x, int y, int br, int h, __int64 style, DiagDaten* data)
{
	LDiag* ret = uiFactory.createLDiag(uiFactory.initParam);
	ret->addStyle(style);
	ret->setPosition(x, y);
	ret->setSize(br, h);
	if (data)
		ret->setDiagDatenZ(data);
	return ret;
}

AuswahlListe* initAuswahlListe(int x, int y, int br, int h, __int64 style, std::initializer_list< char* > values)
{
	AuswahlListe* ret = uiFactory.createAuswahlListe(uiFactory.initParam);
	ret->setPosition(x, y);
	ret->setSize(br, h);
	ret->addStyle(style);
	for (auto i = values.begin(); i != values.end(); i++)
		ret->addEintrag(*i);
	ret->update();
	return ret;
}

ZeichnungHintergrund* initZeichnungHintergrund(int x, int y, int br, int h, __int64 style, int farbe)
{
	ZeichnungHintergrund* ret = new ZeichnungHintergrund();
	ret->setStyle(style);
	ret->setHintergrundFarbe(farbe);
	if (ret->hatStyle(ZeichnungHintergrund::Style::Rahmen))
	{
		ret->setRahmenBreite(1);
		ret->setRahmenFarbe(0xFF00FF00);
	}
	ret->setPosition(x, y);
	ret->setSize(br, h);
	return ret;
}

void initToolTip(Zeichnung* obj, const char* txt)
{
	obj->setToolTipText(txt, uiFactory.initParam.bildschirm, uiFactory.initParam.schrift);
	obj->zToolTip()->addStyle(TextFeld::Style::Mehrzeilig);
}

FBalken* initFBalken(int x, int y, int br, int h, __int64 style)
{
	FBalken* ret = uiFactory.createFBalken(uiFactory.initParam);
	ret->addStyle(style);
	ret->setPosition(x, y);
	ret->setSize(br, h);
	return ret;
}